@@ -32,7 +32,9 @@
 #include <config.h>
 #endif
 
+#include <gio/gunixinputstream.h>
 #include <glib/gi18n.h>
+#include <gtk/gtk.h>
 #include <string.h>
 
 #include <api/na-core-utils.h>
@@ -40,6 +42,7 @@
 
 #include "na-gnome-vfs-uri.h"
 #include "na-selected-info.h"
+#include "na-settings.h"
 #include "na-tokens.h"
 
 /* private class data
@@ -74,7 +77,12 @@ static void      instance_init( GTypeInstance *instance, gpointer klass );
 static void      instance_dispose( GObject *object );
 static void      instance_finalize( GObject *object );
 
+static gchar    *display_output_get_content( int fd );
 static void      execute_action_command( gchar *command, const NAObjectProfile *profile );
+static gchar    *get_command_execution_display_output( const gchar *command );
+static gchar    *get_command_execution_embedded( const gchar *command );
+static gchar    *get_command_execution_normal( const gchar *command );
+static gchar    *get_command_execution_terminal( const gchar *command );
 static gboolean  is_singular_exec( const NATokens *tokens, const gchar *exec );
 static gchar    *parse_singular( const NATokens *tokens, const gchar *input, guint i, gboolean utf8, gboolean quoted );
 static GString  *quote_string( GString *input, const gchar *name, gboolean quoted );
@@ -141,11 +149,12 @@ instance_init( GTypeInstance *instance, gpointer klass )
 	NATokens *self;
 
 	g_return_if_fail( NA_IS_TOKENS( instance ));
-	self = NA_TOKENS( instance );
 
 	g_debug( "%s: instance=%p (%s), klass=%p",
 			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
 
+	self = NA_TOKENS( instance );
+
 	self->private = g_new0( NATokensPrivate, 1 );
 
 	self->private->uris = NULL;
@@ -170,6 +179,7 @@ instance_dispose( GObject *object )
 	NATokens *self;
 
 	g_return_if_fail( NA_IS_TOKENS( object ));
+
 	self = NA_TOKENS( object );
 
 	if( !self->private->dispose_has_run ){
@@ -191,9 +201,10 @@ instance_finalize( GObject *object )
 	NATokens *self;
 
 	g_return_if_fail( NA_IS_TOKENS( object ));
-	self = NA_TOKENS( object );
 
-	g_debug( "%s: object=%p", thisfn, ( void * ) object );
+	g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
+
+	self = NA_TOKENS( object );
 
 	g_free( self->private->scheme );
 	g_free( self->private->username );
@@ -385,45 +396,205 @@ na_tokens_execute_action( const NATokens *tokens, const NAObjectProfile *profile
 	g_free( exec );
 }
 
+static void
+display_output( const gchar *command, int fd_stdout, int fd_stderr )
+{
+	GtkWidget *dialog;
+	gchar *std_output, *std_error;
+
+	dialog = gtk_message_dialog_new_with_markup(
+			NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, "<b>%s</b>", _( "Output of the run command" ));
+	g_object_set( G_OBJECT( dialog ) , "title", PACKAGE_NAME, NULL );
+
+	std_output = display_output_get_content( fd_stdout );
+	std_error = display_output_get_content( fd_stderr );
+
+	gtk_message_dialog_format_secondary_markup( GTK_MESSAGE_DIALOG( dialog ),
+			"<b>%s</b>\n%s\n\n<b>%s</b>\n%s\n\n<b>%s</b>\n%s\n\n",
+					_( "Run command:" ), command,
+					_( "Standard output:" ), std_output,
+					_( "Standard error:" ), std_error );
+
+	gtk_dialog_run( GTK_DIALOG( dialog ));
+	gtk_widget_destroy( dialog );
+
+	g_free( std_output );
+	g_free( std_error );
+}
+
+static gchar *
+display_output_get_content( int fd )
+{
+	static const gchar *thisfn = "na_tokens_display_output_get_content";
+	GInputStream *stream;
+	GString *string;
+	gchar buf[1024];
+	GError *error;
+	gchar *msg;
+
+	string = g_string_new( "" );
+	memset( buf, '\0', sizeof( buf ));
+
+	if( fd > 0 ){
+		stream = g_unix_input_stream_new( fd, TRUE );
+		error = NULL;
+
+		while( g_input_stream_read( stream, buf, sizeof( buf )-1, NULL, &error )){
+			string = g_string_append( string, buf );
+			memset( buf, '\0', sizeof( buf ));
+		}
+		if( error ){
+			g_warning( "%s: g_input_stream_read: %s", thisfn, error->message );
+			g_error_free( error );
+		}
+		g_input_stream_close( stream, NULL, NULL );
+	}
+
+	msg = g_locale_to_utf8( string->str, -1, NULL, NULL, NULL );
+	g_string_free( string, TRUE );
+
+	return( msg );
+}
+
+/*
+ * Execution environment:
+ * - Normal: just execute the specified command
+ * - Terminal: use the user preference to have a terminal which stays openeded
+ * - Embedded: id. Terminal
+ * - DisplayOutput: execute in a shell
+ */
 static void
 execute_action_command( gchar *command, const NAObjectProfile *profile )
 {
 	static const gchar *thisfn = "nautilus_actions_execute_action_command";
 	GError *error;
+	gchar *execution_mode, *run_command;
 	gchar **argv;
 	gint argc;
 	gchar *wdir;
 	GPid child_pid;
+	gint child_stdout, child_stderr;
 
-	g_debug( "%s: command=%s, profile=%p", thisfn, command, ( void * ) profile );
+	g_debug( "%s: profile=%p", thisfn, ( void * ) profile );
 
 	error = NULL;
+	run_command = NULL;
+	execution_mode = na_object_get_execution_mode( profile );
 
-	if( !g_shell_parse_argv( command, &argc, &argv, &error )){
-		g_warning( "%s: %s", thisfn, error->message );
-		g_error_free( error );
+	if( !strcmp( execution_mode, "Normal" )){
+		run_command = get_command_execution_normal( command );
+
+	} else if( !strcmp( execution_mode, "Terminal" )){
+		run_command = get_command_execution_terminal( command );
+
+	} else if( !strcmp( execution_mode, "Embedded" )){
+		run_command = get_command_execution_embedded( command );
+
+	} else if( !strcmp( execution_mode, "DisplayOutput" )){
+		run_command = get_command_execution_display_output( command );
 
 	} else {
-		wdir = na_object_get_working_dir( profile );
-		g_debug( "%s: wdir=%s", thisfn, wdir );
+		g_warning( "%s: unknown execution mode: %s", thisfn, execution_mode );
+	}
 
-		g_spawn_async( wdir,
-				argv,
-				NULL,
-				G_SPAWN_SEARCH_PATH | G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
-				NULL,
-				NULL,
-				&child_pid,
-				&error );
-		if( error ){
-			g_warning( "%s: %s", thisfn, error->message );
+	if( run_command ){
+		if( !g_shell_parse_argv( run_command, &argc, &argv, &error )){
+			g_warning( "%s: g_shell_parse_argv: %s", thisfn, error->message );
 			g_error_free( error );
+
+		} else {
+			wdir = na_object_get_working_dir( profile );
+			g_debug( "%s: run_command=%s, wdir=%s", thisfn, run_command, wdir );
+
+			g_spawn_async_with_pipes( wdir,
+					argv,
+					NULL,
+					G_SPAWN_SEARCH_PATH,
+					NULL,
+					NULL,
+					&child_pid,
+					NULL,
+					&child_stdout,
+					&child_stderr,
+					&error );
+			if( error ){
+				g_warning( "%s: g_spawn_async_with_pipes: %s", thisfn, error->message );
+				g_error_free( error );
+
+			} else {
+				g_spawn_close_pid( child_pid );
+
+				if( !strcmp( execution_mode, "DisplayOutput" )){
+					display_output( run_command, child_stdout, child_stderr );
+				}
+			}
+
+			g_free( wdir );
+			g_strfreev( argv );
 		}
 
-		g_spawn_close_pid( child_pid );
-		g_free( wdir );
-		g_strfreev( argv );
+		g_free( run_command );
 	}
+
+	g_free( execution_mode );
+}
+
+static gchar *
+get_command_execution_display_output( const gchar *command )
+{
+	static const gchar *bin_sh = "/bin/sh -c \"%s\"";
+	return( na_tokens_command_for_terminal( bin_sh, command ));
+}
+
+static gchar *
+get_command_execution_embedded( const gchar *command )
+{
+	return( get_command_execution_terminal( command ));
+}
+
+static gchar *
+get_command_execution_normal( const gchar *command )
+{
+	return( g_strdup( command ));
+}
+
+static gchar *
+get_command_execution_terminal( const gchar *command )
+{
+	gchar *run_command;
+	gchar *pattern;
+
+	pattern = na_settings_get_string( NA_IPREFS_TERMINAL_PATTERN, NULL, NULL );
+	run_command = na_tokens_command_for_terminal( pattern, command );
+	g_free( pattern );
+
+	return( run_command );
+}
+
+/**
+ * na_tokens_command_for_terminal:
+ * @pattern: the command pattern; should include a 'COMMAND' keyword
+ * @command: the command to be actually run in the terminal
+ *
+ * Returns: the command to be run, as a newly allocated string which should
+ * be g_free() by the caller.
+ */
+gchar *
+na_tokens_command_for_terminal( const gchar *pattern, const gchar *command )
+{
+	gchar *run_command;
+	gchar *quoted;
+
+	if( pattern && strlen( pattern )){
+		quoted = g_shell_quote( command );
+		run_command = na_core_utils_str_subst( pattern, "COMMAND", quoted );
+		g_free( quoted );
+
+	} else {
+		run_command = g_strdup( command );
+	}
+
+	return( run_command );
 }
 
 /*
