@@ -38,13 +38,13 @@
 
 /* private class data
  */
-struct BaseBuilderClassPrivate {
+struct _BaseBuilderClassPrivate {
 	void *empty;						/* so that gcc -pedantic is happy */
 };
 
 /* private instance data
  */
-struct BaseBuilderPrivate {
+struct _BaseBuilderPrivate {
 	gboolean  dispose_has_run;
 	GSList   *fnames;
 };
@@ -119,10 +119,11 @@ instance_init( GTypeInstance *instance, gpointer klass )
 	static const gchar *thisfn = "base_builder_instance_init";
 	BaseBuilder *self;
 
+	g_return_if_fail( BASE_IS_BUILDER( instance ));
+
 	g_debug( "%s: instance=%p (%s), klass=%p",
 			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
 
-	g_return_if_fail( BASE_IS_BUILDER( instance ));
 	self = BASE_BUILDER( instance );
 
 	self->private = g_new0( BaseBuilderPrivate, 1 );
@@ -136,12 +137,14 @@ instance_dispose( GObject *instance )
 	static const gchar *thisfn = "base_builder_instance_dispose";
 	BaseBuilder *self;
 
-	g_debug( "%s: instance=%p (%s)", thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ));
 	g_return_if_fail( BASE_IS_BUILDER( instance ));
+
 	self = BASE_BUILDER( instance );
 
 	if( !self->private->dispose_has_run ){
 
+		g_debug( "%s: instance=%p (%s)", thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ));
+
 		self->private->dispose_has_run = TRUE;
 
 		/* chain up to the parent class */
@@ -152,14 +155,16 @@ instance_dispose( GObject *instance )
 }
 
 static void
-instance_finalize( GObject *window )
+instance_finalize( GObject *instance )
 {
 	static const gchar *thisfn = "base_builder_instance_finalize";
 	BaseBuilder *self;
 
-	g_debug( "%s: window=%p", thisfn, ( void * ) window );
-	g_return_if_fail( BASE_IS_BUILDER( window ));
-	self = BASE_BUILDER( window );
+	g_return_if_fail( BASE_IS_BUILDER( instance ));
+
+	g_debug( "%s: instance=%p (%s)", thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ));
+
+	self = BASE_BUILDER( instance );
 
 	na_core_utils_slist_free( self->private->fnames );
 
@@ -167,7 +172,7 @@ instance_finalize( GObject *window )
 
 	/* chain call to parent class */
 	if( G_OBJECT_CLASS( st_parent_class )->finalize ){
-		G_OBJECT_CLASS( st_parent_class )->finalize( window );
+		G_OBJECT_CLASS( st_parent_class )->finalize( instance );
 	}
 }
 
@@ -225,39 +230,6 @@ base_builder_add_from_file( BaseBuilder *builder, const gchar *filename, GError
 	return( ret );
 }
 
-/**
- * base_builder_load_named_toplevel:
- * @builder: this #BaseBuilder object.
- * @name: the name of the searched toplevel window.
- *
- * This function provides a pointer to the toplevel dialog associated
- * with the specified #BaseWindow.
- *
- * Returns: a pointer to the named dialog, or NULL.
- * This pointer is owned by GtkBuilder instance, and must not be
- * g_free() nor g_object_unref() by the caller.
- */
-GtkWindow *
-base_builder_load_named_toplevel( BaseBuilder *builder, const gchar *name )
-{
-	GtkWindow *toplevel = NULL;
-
-	g_return_val_if_fail( BASE_IS_BUILDER( builder ), NULL );
-	g_return_val_if_fail( name, NULL );
-	g_return_val_if_fail( g_utf8_strlen( name, -1 ), NULL );
-
-	if( !builder->private->dispose_has_run ){
-
-		toplevel = GTK_WINDOW( gtk_builder_get_object( GTK_BUILDER( builder ), name ));
-
-		if( toplevel ){
-			g_return_val_if_fail( GTK_IS_WINDOW( toplevel ), NULL );
-		}
-	}
-
-	return( toplevel );
-}
-
 static gboolean
 already_loaded( BaseBuilder *builder, const gchar *filename )
 {
@@ -272,3 +244,39 @@ already_loaded( BaseBuilder *builder, const gchar *filename )
 
 	return( loaded );
 }
+
+/**
+ * base_builder_get_toplevel_by_name:
+ * @builder: this #BaseBuilder object.
+ * @name: the name of the searched toplevel window.
+ *
+ * This function provides a pointer to the toplevel dialog associated
+ * with the specified #BaseWindow.
+ *
+ * Returns: a pointer to the named dialog, or NULL.
+ * This pointer is owned by GtkBuilder instance, and must not be
+ * g_free() nor g_object_unref() by the caller.
+ */
+GtkWindow *
+base_builder_get_toplevel_by_name( const BaseBuilder *builder, const gchar *name )
+{
+	static const gchar *thisfn = "base_builder_get_toplevel_by_name";
+	GtkWindow *toplevel = NULL;
+
+	g_return_val_if_fail( BASE_IS_BUILDER( builder ), NULL );
+	g_return_val_if_fail( name, NULL );
+	g_return_val_if_fail( g_utf8_strlen( name, -1 ), NULL );
+
+	if( !builder->private->dispose_has_run ){
+
+		toplevel = GTK_WINDOW( gtk_builder_get_object( GTK_BUILDER( builder ), name ));
+
+		if( toplevel ){
+			g_return_val_if_fail( GTK_IS_WINDOW( toplevel ), NULL );
+		} else {
+			na_core_utils_slist_dump( thisfn, builder->private->fnames );
+		}
+	}
+
+	return( toplevel );
+}
