@@ -42,11 +42,10 @@
 
 #include <api/na-core-utils.h>
 #include <api/na-object-api.h>
+#include <api/na-timeout.h>
 
 #include <core/na-pivot.h>
-#include <core/na-iabout.h>
-#include <core/na-iprefs.h>
-#include <core/na-ipivot-consumer.h>
+#include <core/na-about.h>
 #include <core/na-selected-info.h>
 #include <core/na-tokens.h>
 
@@ -54,19 +53,23 @@
 
 /* private class data
  */
-struct NautilusActionsClassPrivate {
+struct _NautilusActionsClassPrivate {
 	void *empty;						/* so that gcc -pedantic is happy */
 };
 
 /* private instance data
  */
-struct NautilusActionsPrivate {
-	gboolean dispose_has_run;
-	NAPivot *pivot;
+struct _NautilusActionsPrivate {
+	gboolean  dispose_has_run;
+	NAPivot  *pivot;
+	gulong    items_changed_handler;
+	gulong    settings_changed_handler;
+	NATimeout change_timeout;
 };
 
-static GObjectClass *st_parent_class = NULL;
-static GType         st_actions_type = 0;
+static GObjectClass *st_parent_class  = NULL;
+static GType         st_actions_type  = 0;
+static gint          st_burst_timeout = 100;		/* burst timeout in msec */
 
 static void              class_init( NautilusActionsClass *klass );
 static void              instance_init( GTypeInstance *instance, gpointer klass );
@@ -74,19 +77,13 @@ static void              instance_constructed( GObject *object );
 static void              instance_dispose( GObject *object );
 static void              instance_finalize( GObject *object );
 
-static void              iabout_iface_init( NAIAboutInterface *iface );
-static gchar            *iabout_get_application_name( NAIAbout *instance );
-
-static void              ipivot_consumer_iface_init( NAIPivotConsumerInterface *iface );
-static void              ipivot_consumer_items_changed( NAIPivotConsumer *instance, gpointer user_data );
-static void              ipivot_consumer_create_root_menu_changed( NAIPivotConsumer *instance, gboolean enabled );
-static void              ipivot_consumer_display_about_changed( NAIPivotConsumer *instance, gboolean enabled );
-static void              ipivot_consumer_display_order_changed( NAIPivotConsumer *instance, gint order_mode );
-
 static void              menu_provider_iface_init( NautilusMenuProviderIface *iface );
 static GList            *menu_provider_get_background_items( NautilusMenuProvider *provider, GtkWidget *window, NautilusFileInfo *current_folder );
 static GList            *menu_provider_get_file_items( NautilusMenuProvider *provider, GtkWidget *window, GList *files );
+
+#ifdef HAVE_NAUTILUS_MENU_PROVIDER_GET_TOOLBAR_ITEMS
 static GList            *menu_provider_get_toolbar_items( NautilusMenuProvider *provider, GtkWidget *window, NautilusFileInfo *current_folder );
+#endif
 
 static GList            *get_menus_items( NautilusActions *plugin, guint target, GList *selection );
 static GList            *expand_tokens( GList *tree, NATokens *tokens );
@@ -106,6 +103,10 @@ static GList            *create_root_menu( NautilusActions *plugin, GList *nauti
 static GList            *add_about_item( NautilusActions *plugin, GList *nautilus_menu );
 static void              execute_about( NautilusMenuItem *item, NautilusActions *plugin );
 
+static void              on_pivot_items_changed_handler( NAPivot *pivot, NautilusActions *plugin );
+static void              on_settings_key_changed_handler( const gchar *group, const gchar *key, gconstpointer new_value, gboolean mandatory, NautilusActions *plugin );
+static void              on_change_event_timeout( NautilusActions *plugin );
+
 GType
 nautilus_actions_get_type( void )
 {
@@ -136,28 +137,13 @@ nautilus_actions_register_type( GTypeModule *module )
 		NULL
 	};
 
-	static const GInterfaceInfo iabout_iface_info = {
-		( GInterfaceInitFunc ) iabout_iface_init,
-		NULL,
-		NULL
-	};
-
-	static const GInterfaceInfo ipivot_consumer_iface_info = {
-		( GInterfaceInitFunc ) ipivot_consumer_iface_init,
-		NULL,
-		NULL
-	};
+	g_assert( st_actions_type == 0 );
 
 	g_debug( "%s: module=%p", thisfn, ( void * ) module );
-	g_assert( st_actions_type == 0 );
 
 	st_actions_type = g_type_module_register_type( module, G_TYPE_OBJECT, "NautilusActions", &info, 0 );
 
 	g_type_module_add_interface( module, st_actions_type, NAUTILUS_TYPE_MENU_PROVIDER, &menu_provider_iface_info );
-
-	g_type_module_add_interface( module, st_actions_type, NA_IABOUT_TYPE, &iabout_iface_info );
-
-	g_type_module_add_interface( module, st_actions_type, NA_IPIVOT_CONSUMER_TYPE, &ipivot_consumer_iface_info );
 }
 
 static void
@@ -184,40 +170,75 @@ instance_init( GTypeInstance *instance, gpointer klass )
 	static const gchar *thisfn = "nautilus_actions_instance_init";
 	NautilusActions *self;
 
-	g_debug( "%s: instance=%p, klass=%p", thisfn, ( void * ) instance, ( void * ) klass );
 	g_return_if_fail( NAUTILUS_IS_ACTIONS( instance ));
-	g_return_if_fail( NA_IS_IPIVOT_CONSUMER( instance ));
+
+	g_debug( "%s: instance=%p (%s), klass=%p",
+			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
 
 	self = NAUTILUS_ACTIONS( instance );
 
 	self->private = g_new0( NautilusActionsPrivate, 1 );
 
 	self->private->dispose_has_run = FALSE;
-
-	na_ipivot_consumer_allow_notify( NA_IPIVOT_CONSUMER( instance ), TRUE, 0 );
+	self->private->change_timeout.timeout = st_burst_timeout;
+	self->private->change_timeout.handler = ( NATimeoutFunc ) on_change_event_timeout;
+	self->private->change_timeout.user_data = self;
+	self->private->change_timeout.source_id = 0;
 }
 
+/*
+ * Runtime modification management:
+ * We have to react to some runtime environment modifications:
+ *
+ * - whether the items list has changed (we have to reload a new pivot)
+ *   > registering for notifications against NAPivot
+ *
+ * - whether to add the 'About Nautilus-Actions' item
+ * - whether to create a 'Nautilus-Actions actions' root menu
+ *   > registering for notifications against NASettings
+ */
 static void
 instance_constructed( GObject *object )
 {
 	static const gchar *thisfn = "nautilus_actions_instance_constructed";
 	NautilusActions *self;
 
-	g_debug( "%s: object=%p", thisfn, ( void * ) object );
-
 	g_return_if_fail( NAUTILUS_IS_ACTIONS( object ));
-	g_return_if_fail( NA_IS_IPIVOT_CONSUMER( object ));
 
 	self = NAUTILUS_ACTIONS( object );
 
 	if( !self->private->dispose_has_run ){
+		g_debug( "%s: object=%p", thisfn, ( void * ) object );
 
 		self->private->pivot = na_pivot_new();
-		na_pivot_register_consumer( self->private->pivot, NA_IPIVOT_CONSUMER( self ));
-		na_pivot_set_automatic_reload( self->private->pivot, TRUE );
+
+		/* setup NAPivot properties before loading items
+		 */
 		na_pivot_set_loadable( self->private->pivot, !PIVOT_LOAD_DISABLED & !PIVOT_LOAD_INVALID );
 		na_pivot_load_items( self->private->pivot );
 
+		/* register against NAPivot to be notified of items changes
+		 */
+		self->private->items_changed_handler =
+				g_signal_connect( self->private->pivot,
+						PIVOT_SIGNAL_ITEMS_CHANGED, G_CALLBACK( on_pivot_items_changed_handler ), self );
+
+		/* register against NASettings to be notified of changes on
+		 *  our runtime preferences
+		 * because we only monitor here four runtime keys, we prefer the
+		 * callback way that the signal one
+		 */
+		na_settings_register_key_callback(
+				NA_IPREFS_IO_PROVIDERS_READ_STATUS, G_CALLBACK( on_settings_key_changed_handler ), self );
+		na_settings_register_key_callback(
+				NA_IPREFS_ITEMS_ADD_ABOUT_ITEM, G_CALLBACK( on_settings_key_changed_handler ), self );
+		na_settings_register_key_callback(
+				NA_IPREFS_ITEMS_CREATE_ROOT_MENU, G_CALLBACK( on_settings_key_changed_handler ), self );
+		na_settings_register_key_callback(
+				NA_IPREFS_ITEMS_LEVEL_ZERO_ORDER, G_CALLBACK( on_settings_key_changed_handler ), self );
+		na_settings_register_key_callback(
+				NA_IPREFS_ITEMS_LIST_ORDER_MODE, G_CALLBACK( on_settings_key_changed_handler ), self );
+
 		/* chain up to the parent class */
 		if( G_OBJECT_CLASS( st_parent_class )->constructed ){
 			G_OBJECT_CLASS( st_parent_class )->constructed( object );
@@ -239,6 +260,9 @@ instance_dispose( GObject *object )
 
 		self->private->dispose_has_run = TRUE;
 
+		if( self->private->items_changed_handler ){
+			g_signal_handler_disconnect( self->private->pivot, self->private->items_changed_handler );
+		}
 		g_object_unref( self->private->pivot );
 
 		/* chain up to the parent class */
@@ -266,124 +290,10 @@ instance_finalize( GObject *object )
 	}
 }
 
-/*
- * This function notifies Nautilus file manager that the context menu
- * items may have changed, and that it should reload them.
- *
- * Patch has been provided by Frederic Ruaudel, the initial author of
- * Nautilus-Actions, and applied on Nautilus 2.15.4 development branch
- * on 2006-06-16. It was released with Nautilus 2.16.0
- */
-#ifndef HAVE_NAUTILUS_MENU_PROVIDER_EMIT_ITEMS_UPDATED_SIGNAL
-static void nautilus_menu_provider_emit_items_updated_signal (NautilusMenuProvider *provider)
+gboolean
+nautilus_action_is_log_enabled( void )
 {
-	/* -> fake function for backward compatibility
-	 * -> do nothing
-	 */
-}
-#endif
-
-static void
-iabout_iface_init( NAIAboutInterface *iface )
-{
-	static const gchar *thisfn = "nautilus_actions_iabout_iface_init";
-
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
-
-	iface->get_application_name = iabout_get_application_name;
-}
-
-static gchar *
-iabout_get_application_name( NAIAbout *instance )
-{
-	/* i18n: title of the About dialog box, when seen from Nautilus file manager */
-	return( g_strdup( _( "Nautilus-Actions" )));
-}
-
-static void
-ipivot_consumer_iface_init( NAIPivotConsumerInterface *iface )
-{
-	static const gchar *thisfn = "nautilus_actions_ipivot_consumer_iface_init";
-
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
-
-	iface->on_items_changed = ipivot_consumer_items_changed;
-	iface->on_create_root_menu_changed = ipivot_consumer_create_root_menu_changed;
-	iface->on_display_about_changed = ipivot_consumer_display_about_changed;
-	iface->on_display_order_changed = ipivot_consumer_display_order_changed;
-	iface->on_mandatory_prefs_changed = NULL;
-}
-
-static void
-ipivot_consumer_items_changed( NAIPivotConsumer *instance, gpointer user_data )
-{
-	static const gchar *thisfn = "nautilus_actions_ipivot_consumer_items_changed";
-	NautilusActions *self;
-
-	g_return_if_fail( NAUTILUS_IS_ACTIONS( instance ));
-
-	self = NAUTILUS_ACTIONS( instance );
-
-	if( !self->private->dispose_has_run ){
-
-		g_debug( "%s: instance=%p, user_data=%p", thisfn, ( void * ) instance, ( void * ) user_data );
-
-		nautilus_menu_provider_emit_items_updated_signal( NAUTILUS_MENU_PROVIDER( self ));
-	}
-}
-
-static void
-ipivot_consumer_create_root_menu_changed( NAIPivotConsumer *instance, gboolean enabled )
-{
-	static const gchar *thisfn = "nautilus_actions_ipivot_consumer_create_root_menu_changed";
-	NautilusActions *self;
-
-	g_return_if_fail( NAUTILUS_IS_ACTIONS( instance ));
-
-	self = NAUTILUS_ACTIONS( instance );
-
-	if( !self->private->dispose_has_run ){
-
-		g_debug( "%s: instance=%p, enabled=%s", thisfn, ( void * ) instance, enabled ? "True":"False" );
-
-		nautilus_menu_provider_emit_items_updated_signal( NAUTILUS_MENU_PROVIDER( self ));
-	}
-}
-
-static void
-ipivot_consumer_display_about_changed( NAIPivotConsumer *instance, gboolean enabled )
-{
-	static const gchar *thisfn = "nautilus_actions_ipivot_consumer_display_about_changed";
-	NautilusActions *self;
-
-	g_return_if_fail( NAUTILUS_IS_ACTIONS( instance ));
-
-	self = NAUTILUS_ACTIONS( instance );
-
-	if( !self->private->dispose_has_run ){
-
-		g_debug( "%s: instance=%p, enabled=%s", thisfn, ( void * ) instance, enabled ? "True":"False" );
-
-		nautilus_menu_provider_emit_items_updated_signal( NAUTILUS_MENU_PROVIDER( self ));
-	}
-}
-
-static void
-ipivot_consumer_display_order_changed( NAIPivotConsumer *instance, gint order_mode )
-{
-	static const gchar *thisfn = "nautilus_actions_ipivot_consumer_display_order_changed";
-	NautilusActions *self;
-
-	g_return_if_fail( NAUTILUS_IS_ACTIONS( instance ));
-
-	self = NAUTILUS_ACTIONS( instance );
-
-	if( !self->private->dispose_has_run ){
-
-		g_debug( "%s: instance=%p, order_mode=%d", thisfn, ( void * ) instance, order_mode );
-
-		nautilus_menu_provider_emit_items_updated_signal( NAUTILUS_MENU_PROVIDER( self ));
-	}
+	return( na_settings_get_boolean( NA_IPREFS_PLUGIN_LOG, NULL, NULL ));
 }
 
 static void
@@ -395,7 +305,10 @@ menu_provider_iface_init( NautilusMenuProviderIface *iface )
 
 	iface->get_file_items = menu_provider_get_file_items;
 	iface->get_background_items = menu_provider_get_background_items;
+
+#ifdef HAVE_NAUTILUS_MENU_PROVIDER_GET_TOOLBAR_ITEMS
 	iface->get_toolbar_items = menu_provider_get_toolbar_items;
+#endif
 }
 
 /*
@@ -488,10 +401,13 @@ menu_provider_get_file_items( NautilusMenuProvider *provider, GtkWidget *window,
 	return( nautilus_menus_list );
 }
 
+#ifdef HAVE_NAUTILUS_MENU_PROVIDER_GET_TOOLBAR_ITEMS
 /*
  * as of 2.26, this function is only called for folders, but for the
  * desktop (x-nautilus-desktop:///) which seems to be only called by
  * get_background_items ; also, only actions (not menus) are displayed
+ *
+ * the API is removed starting with Nautilus 2.91.90
  */
 static GList *
 menu_provider_get_toolbar_items( NautilusMenuProvider *provider, GtkWidget *window, NautilusFileInfo *current_folder )
@@ -520,6 +436,7 @@ menu_provider_get_toolbar_items( NautilusMenuProvider *provider, GtkWidget *wind
 
 	return( nautilus_menus_list );
 }
+#endif
 
 static GList *
 get_menus_items( NautilusActions *plugin, guint target, GList *selection )
@@ -527,8 +444,8 @@ get_menus_items( NautilusActions *plugin, guint target, GList *selection )
 	GList *menus_list;
 	NATokens *tokens;
 	GList *pivot_tree, *copy_tree;
-	gboolean root_menu;
-	gboolean add_about;
+	gboolean items_add_about_item;
+	gboolean items_create_root_menu;
 
 	g_return_val_if_fail( NA_IS_PIVOT( plugin->private->pivot ), NULL );
 
@@ -538,18 +455,18 @@ get_menus_items( NautilusActions *plugin, guint target, GList *selection )
 
 	menus_list = build_nautilus_menus( plugin, copy_tree, target, selection, tokens );
 
-	na_object_unref_items( copy_tree );
+	na_object_free_items( copy_tree );
 	g_object_unref( tokens );
 
 	if( target != ITEM_TARGET_TOOLBAR ){
 
-		root_menu = na_iprefs_read_bool( NA_IPREFS( plugin->private->pivot ), IPREFS_CREATE_ROOT_MENU, FALSE );
-		if( root_menu ){
+		items_create_root_menu = na_settings_get_boolean( NA_IPREFS_ITEMS_CREATE_ROOT_MENU, NULL, NULL );
+		if( items_create_root_menu ){
 			menus_list = create_root_menu( plugin, menus_list );
 		}
 
-		add_about = na_iprefs_read_bool( NA_IPREFS( plugin->private->pivot ), IPREFS_ADD_ABOUT_ITEM, TRUE );
-		if( add_about ){
+		items_add_about_item = na_settings_get_boolean( NA_IPREFS_ITEMS_ADD_ABOUT_ITEM, NULL, NULL );
+		if( items_add_about_item ){
 			menus_list = add_about_item( plugin, menus_list );
 		}
 	}
@@ -560,7 +477,7 @@ get_menus_items( NautilusActions *plugin, guint target, GList *selection )
 /*
  * create a copy of the tree where almost all fields which may embed
  * parameters have been expanded
- * here, 'almost' should be readen as:
+ * here, 'almost' should be read as:
  * - all displayable fields, or fields which may have an impact on the display
  *   (e.g. label, tooltip, icon name)
  * - all fields which we do not need later
@@ -969,7 +886,7 @@ create_root_menu( NautilusActions *plugin, GList *menu )
 		return( NULL );
 	}
 
-	icon = na_iabout_get_icon_name();
+	icon = na_about_get_icon_name();
 	root_item = nautilus_menu_item_new( "NautilusActionsExtensions",
 				/* i18n: label of an automagic root submenu */
 				_( "Nautilus-Actions actions" ),
@@ -1018,7 +935,7 @@ add_about_item( NautilusActions *plugin, GList *menu )
 	}
 
 	if( have_root_menu ){
-		icon = na_iabout_get_icon_name();
+		icon = na_about_get_icon_name();
 
 		about_item = nautilus_menu_item_new( "AboutNautilusActions",
 					_( "About Nautilus-Actions" ),
@@ -1044,7 +961,60 @@ static void
 execute_about( NautilusMenuItem *item, NautilusActions *plugin )
 {
 	g_return_if_fail( NAUTILUS_IS_ACTIONS( plugin ));
-	g_return_if_fail( NA_IS_IABOUT( plugin ));
 
-	na_iabout_display( NA_IABOUT( plugin ));
+	na_about_display( NULL );
+}
+
+/*
+ * Not only the items list itself, but also several runtime preferences have
+ * an effect on the display of items in file manager context menu.
+ *
+ * We of course monitor here all these informations; only asking NAPivot
+ * for reloading its items when we detect the end of a burst of changes.
+ *
+ * Only when NAPivot has finished with reloading its items list, then we
+ * inform the file manager that its items list has changed.
+ */
+
+/* signal emitted by NAPivot at the end of a burst of 'item-changed' signals
+ * from i/o providers
+ */
+static void
+on_pivot_items_changed_handler( NAPivot *pivot, NautilusActions *plugin )
+{
+	g_return_if_fail( NA_IS_PIVOT( pivot ));
+	g_return_if_fail( NAUTILUS_IS_ACTIONS( plugin ));
+
+	if( !plugin->private->dispose_has_run ){
+
+		na_timeout_event( &plugin->private->change_timeout );
+	}
+}
+
+/* callback triggered by NASettings at the end of a burst of 'changed' signals
+ * on runtime preferences which may affect the way file manager displays
+ * its context menus
+ */
+static void
+on_settings_key_changed_handler( const gchar *group, const gchar *key, gconstpointer new_value, gboolean mandatory, NautilusActions *plugin )
+{
+	g_return_if_fail( NAUTILUS_IS_ACTIONS( plugin ));
+
+	if( !plugin->private->dispose_has_run ){
+
+		na_timeout_event( &plugin->private->change_timeout );
+	}
+}
+
+/*
+ * automatically reloads the items, then signal the file manager.
+ */
+static void
+on_change_event_timeout( NautilusActions *plugin )
+{
+	static const gchar *thisfn = "nautilus_actions_on_change_event_timeout";
+	g_debug( "%s: timeout expired", thisfn );
+
+	na_pivot_load_items( plugin->private->pivot );
+	nautilus_menu_provider_emit_items_updated_signal( NAUTILUS_MENU_PROVIDER( plugin ));
 }
